@@ -442,16 +442,15 @@ class WordType(caching.base.CachingMixin,models.Model):
     def getFreq(self,subset='all',speaker=None,dialog=None):
         if subset=='all' and self.Frequency is not None:
             return self.Frequency
+        base = WordToken.objects.all().select_related('WordType','Category')
         if subset == 'gender' and speaker is not None:
-            base = WordToken.objects.filter(Dialog__Speaker__Gender = speaker.Gender)
+            base = base.filter(Dialog__Speaker__Gender = speaker.Gender)
         elif subset == 'age' and speaker is not None:
-            base = WordToken.objects.filter(Dialog__Speaker__Age = speaker.Age)
+            base = base.filter(Dialog__Speaker__Age = speaker.Age)
         elif subset == 'speaker' and speaker is not None:
-            base = WordToken.objects.filter(Dialog__Speaker = speaker)
+            base = base.filter(Dialog__Speaker = speaker)
         elif subset == 'dialog' and dialog is not None:
-            base = WordToken.objects.filter(Dialog = dialog)
-        else:
-            base = WordToken.objects.all()
+            base = base.filter(Dialog = dialog)
 
         Freq = float(base.filter(WordType=self).count())/float(base.filter(~Q(Category__CategoryType='Other'),~Q(Category__CategoryType='Pause'),~Q(Category__CategoryType='Disfluency')).count())
         Freq = math.log((Freq * 1000000)+1,10)
@@ -504,43 +503,59 @@ class WordType(caching.base.CachingMixin,models.Model):
     def getPhonoProb(self,subset='all',speaker=None,dialog=None):
         if subset == 'all' and self.SPhonoProb is not None and self.BiPhonoProb is not None:
             return self.SPhonoProb,self.BiPhonoProb
+        base = WordType.objects.filter(Label__regex="^[^{<]")
+        if subset == 'gender' and speaker is not None:
+            base = base.filter(wordtoken__Dialog__Speaker__Gender = speaker.Gender)
+        elif subset == 'age' and speaker is not None:
+            base = base.filter(wordtoken__Dialog__Speaker__Age = speaker.Age)
+        elif subset == 'speaker' and speaker is not None:
+            base = base.filter(wordtoken__Dialog__Speaker = speaker)
+        elif subset == 'dialog' and dialog is not None:
+            base = base.filter(wordtoken__Dialog = dialog)
+        base = base.annotate(num = Count('wordtoken'))
         any_segment = '[A-Za-z]{1,2}'
         patterns = []
         SPprob = 0.0
         BPprob = 0.0
         phones = map(str,self.UR.all())
+        allwords = float(base.aggregate(totcount=Sum('num'))['totcount'])
         for i in range(len(phones)):
             patt = [any_segment] * i
             patt.append(phones[i])
             pattern = '^'+' '.join(patt) +'.*$'
             totPattern = '^'+' '.join([any_segment] * (i+1)) +'.*$'
-            count = WordType.objects.filter(Label__regex="^[^{<]").extra(
+            count = base.extra(
                     where = [UR_LOOKUP],
                     params = [pattern])
-            totCount = WordType.objects.filter(Label__regex="^[^{<]").extra(
+            totCount = base.extra(
                     where = [UR_LOOKUP],
                     params = [totPattern])
-            SPprob += float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
-                                for x in count])) / float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
-                                                                    for x in totCount]))
+            countFreqs = [math.log(((float(x.num)/allwords) * 1000000)+1,10)
+                                for x in count]
+            totFreqs = [math.log(((float(x.num)/allwords) * 1000000)+1,10)
+                                for x in totCount]
+            SPprob += float(sum(countFreqs)) / float(sum(totFreqs))
             if i != len(phones)-1:
                 patt = [any_segment] * i
                 patt.extend([phones[i],phones[i+1]])
                 pattern = '^'+' '.join(patt) +'.*$'
                 totPattern = '^'+' '.join([any_segment] * (i+2)) +'.*$'
-                count = WordType.objects.filter(Label__regex="^[^{<]").extra(
+                count = base.extra(
                     where = [UR_LOOKUP],
                     params = [pattern])
-                totCount = WordType.objects.filter(Label__regex="^[^{<]").extra(
+                totCount = base.extra(
                     where = [UR_LOOKUP],
                     params = [totPattern])
-                BPprob += float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
-                                for x in count])) / float(sum([x.getFreq(subset=subset,speaker=speaker,dialog=dialog)
-                                                                    for x in totCount]))
+                countFreqs = [math.log(((float(x.num)/allwords) * 1000000)+1,10)
+                                for x in count]
+                totFreqs = [math.log(((float(x.num)/allwords) * 1000000)+1,10)
+                                for x in totCount]
+                BPprob += float(sum(countFreqs)) / float(sum(totFreqs))
         SPprob = SPprob / float(len(phones))
         BPprob = BPprob / float(len(phones)-1)
-        self.SPhonoProb,self.BiPhonoProb = SPprob,BPprob
-        self.save()
+        if subset == 'all':
+            self.SPhonoProb,self.BiPhonoProb = SPprob,BPprob
+            self.save()
         return SPprob,BPprob
 
 
